@@ -26,12 +26,13 @@
 #include "grn_index_column.h"
 #include "grn_pat.h"
 #include "grn_store.h"
+#include "grn_token_column.h"
 
-grn_bool
+bool
 grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
@@ -39,41 +40,34 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
     switch (obj->header.domain) {
     case GRN_DB_BOOL :
       return GRN_BOOL_VALUE(obj);
-      break;
     case GRN_DB_INT32 :
       return GRN_INT32_VALUE(obj) != 0;
-      break;
     case GRN_DB_UINT32 :
       return GRN_UINT32_VALUE(obj) != 0;
-      break;
     case GRN_DB_FLOAT32 : {
       float float_value;
       float_value = GRN_FLOAT32_VALUE(obj);
       return grn_float32_is_zero(float_value);
-      break;
     }
     case GRN_DB_FLOAT : {
       double float_value;
       float_value = GRN_FLOAT_VALUE(obj);
       return grn_float_is_zero(float_value);
-      break;
     }
     case GRN_DB_SHORT_TEXT :
     case GRN_DB_TEXT :
     case GRN_DB_LONG_TEXT :
       return GRN_TEXT_LEN(obj) != 0;
-      break;
     default :
-      return GRN_FALSE;
-      break;
+      return false;
     }
     break;
   case GRN_VECTOR :
-    return GRN_TRUE;
-    break;
+    return true;
+  case GRN_UVECTOR :
+    return true;
   default :
-    return  GRN_FALSE;
-    break;
+    return false;
   }
 }
 
@@ -203,27 +197,102 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
   return is_table;
 }
 
-grn_bool
-grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj)
+bool
+grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_lexicon = GRN_FALSE;
+  grn_bool is_table_with_key = false;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
   case GRN_TABLE_HASH_KEY :
   case GRN_TABLE_PAT_KEY :
   case GRN_TABLE_DAT_KEY :
-    is_lexicon = GRN_TRUE;
+    is_table_with_key = true;
   default :
     break;
   }
 
+  return is_table_with_key;
+}
+
+bool
+grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_table_with_key(ctx, obj)) {
+    return false;
+  }
+
+  grn_hash *columns = grn_hash_create(ctx,
+                                      NULL,
+                                      sizeof(grn_id),
+                                      0,
+                                      GRN_OBJ_TABLE_HASH_KEY | GRN_HASH_TINY);
+  if (!columns) {
+    return false;
+  }
+  grn_table_columns(ctx, obj, "", 0, (grn_obj *)columns);
+  bool is_lexicon = false;
+  if (grn_hash_size(ctx, columns) > 0) {
+    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+      void *key;
+      grn_hash_cursor_get_key(ctx, cursor, &key);
+      grn_id column_id = *((grn_id *)key);
+      grn_obj *column = grn_ctx_at(ctx, column_id);
+      if (grn_obj_is_index_column(ctx, column)) {
+        is_lexicon = true;
+      }
+      grn_obj_unref(ctx, column);
+      if (is_lexicon) {
+        break;
+      }
+    } GRN_HASH_EACH_END(ctx, cursor);
+  }
+  grn_hash_close(ctx, columns);
   return is_lexicon;
 }
 
+bool
+grn_obj_is_lexicon_without_data_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_table_with_key(ctx, obj)) {
+    return false;
+  }
+
+  grn_hash *columns = grn_hash_create(ctx,
+                                      NULL,
+                                      sizeof(grn_id),
+                                      0,
+                                      GRN_OBJ_TABLE_HASH_KEY | GRN_HASH_TINY);
+  if (!columns) {
+    return false;
+  }
+  grn_table_columns(ctx, obj, "", 0, (grn_obj *)columns);
+  bool is_lexicon_without_data_column;
+  if (grn_hash_size(ctx, columns) == 0) {
+    is_lexicon_without_data_column = false;
+  } else {
+    is_lexicon_without_data_column = true;
+    GRN_HASH_EACH_BEGIN(ctx, columns, cursor, id) {
+      void *key;
+      grn_hash_cursor_get_key(ctx, cursor, &key);
+      grn_id column_id = *((grn_id *)key);
+      grn_obj *column = grn_ctx_at(ctx, column_id);
+      if (!grn_obj_is_index_column(ctx, column)) {
+        is_lexicon_without_data_column = false;
+      }
+      grn_obj_unref(ctx, column);
+      if (!is_lexicon_without_data_column) {
+        break;
+      }
+    } GRN_HASH_EACH_END(ctx, cursor);
+  }
+  grn_hash_close(ctx, columns);
+  return is_lexicon_without_data_column;
+}
+
 grn_bool
 grn_obj_is_column(grn_ctx *ctx, grn_obj *obj)
 {
@@ -255,6 +324,16 @@ grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj)
   return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_SCALAR;
 }
 
+bool
+grn_obj_is_text_family_scalar_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_scalar_column(ctx, obj)) {
+    return false;
+  }
+
+  return grn_type_id_is_text_family(ctx, grn_obj_get_range(ctx, obj));
+}
+
 bool
 grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj)
 {
@@ -885,6 +964,10 @@ grn_data_column_reindex(grn_ctx *ctx, grn_obj *data_column)
 {
   grn_hook *hooks;
 
+  if (grn_obj_is_token_column(ctx, data_column)) {
+    grn_token_column_build(ctx, data_column);
+  }
+
   for (hooks = DB_OBJ(data_column)->hooks[GRN_HOOK_SET];
        hooks;
        hooks = hooks->next) {
